#include <ibsys.h>


/*
 * Enable memory on the host CPU for access by the DMA controller.
 * Depending on the host system configuration, this function may
 * never be required.
 */
IBLCL osEnabMem()
{
	/* NULL function on PC/AT machines */
}


/*
 * Allocate any system DMA resources, as necessary, and
 * convert local virtual address to suitable physical
 * address for use by the DMA controller.
 */
IBLCL osDMAalloc(rwop) ibio_op_t *rwop;
{
	int		s;

	DBGin("osDMAalloc");
	rwop->io_pbuf = (uint32) rwop->io_vbuf;
	DBGout();
}


/*
 * Release any previously allocated system DMA resources,
 * clear the memory cache, etc.
 */
IBLCL osDMArelse(rwop) ibio_op_t *rwop;
{
	int		s;

	DBGin("osDMArelse");
        /* the memory is never released under linux */
	DBGout();
}

#if DMAOP
/*
 * Start a DMA operation and wait on its completion.
 */
IBLCL osDoDMA(rwop) ibio_op_t *rwop;
{
	int		resid;		/* residual transfer count */
	int		cnt;		/* word transfer count */
	char		lsb, msb, page;	/* physical address pieces */


	DBGin("osDoDMA");
	osDMAalloc(rwop);
	DBGprint(DBG_DATA, ("pbuf=0x%x cnt=%d  ", rwop->io_pbuf, rwop->io_cnt));


	/* program dma controller */

	cli();
        disable_dma( ibdma );
	clear_dma_ff ( ibdma );
	set_dma_count( ibdma, rwop->io_cnt );
	set_dma_addr ( ibdma, rwop->io_pbuf);

	if (rwop->io_flags & IO_READ) {
		DBGprint(DBG_BRANCH, ("enabling DMA READ  "));
		set_dma_mode( ibdma, DMA_MODE_READ );
	}
	else {
		DBGprint(DBG_BRANCH, ("enabling DMA WRITE  "));
		set_dma_mode( ibdma, DMA_MODE_WRITE );
	}

	GPIBout(dmaEn, (GPIBin(dmaEn) | HR_DMAEN));
	enable_dma( ibdma );/* enable DMA transfers */
	sti();

	bdDMAstart(rwop);
	bdDMAwait(rwop, 0);
	resid = bdDMAstop(rwop);

        disable_dma( ibdma );/* disable DMA transfers */
	GPIBout(dmaEn, (GPIBin(dmaEn) & ~HR_DMAEN));

	osDMArelse(rwop);
	DBGout();
	return resid;
}
#endif

#ifdef HAS_DMAMANAGER
/* Return a pointer to a DMA buffer */
IBLCL char *getdmabuffer( int *size )
{
	char *buf;
	
	if (gpib_dma_buffer) {
		/* Use a pre-allocated DMA buffer */
		buf = gpib_dma_buffer;
		*size = gpib_dma_size;
	} else {
		/* No pre-allocated buffer - allocate one dynamically */
		if (*size > MAX_DMA_SIZE)
			*size = MAX_DMA_SIZE;
#ifdef CONFIG_DMA_MEM_PERM
		buf = (char *)dma_mem_alloc( *size,
					     DMA_CEIL_16M |
					     DMA_SEG_128K |
					     DMA_ALIGN_2);
		/* If we could not get a buffer - try smaller size */
		while ((buf == NULL) && ((*size >>= 1) > 1024)) {
			buf = (char *)dma_mem_alloc( *size,
						     DMA_CEIL_16M |
						     DMA_SEG_128K |
						     DMA_ALIGN_2);
		}
#else
		buf = (char *)kmalloc( *size, MEMORY_PRIORITY );
#endif
		
	}

	return buf;
}

/* Free a DMA buffer if it was dynamically allocated */
IBLCL void freedmabuffer( char *buf )
{
	if (!gpib_dma_buffer) {
#ifdef CONFIG_DMA_MEM_PERM
		dma_mem_free( (long) buf );
#else
		kfree( buf );
#endif
	}
}

#endif /* HAS_DMAMANAGER */

