#include <ibsys.h>

#define GIVE_UP(a) { osUnlockMutex(); DBGout(); return (a); }

#define LINUX_STD_PARAM  struct inode* inode,struct file* filep

int ib_opened=0;

ibopen(LINUX_STD_PARAM)
{
	DBGin("ibopen");

#if CONFIG_EXCLUSIVE
	if (ib_opened) {
		return (-EBUSY);
	}
#endif

	MOD_INC_USE_COUNT;
	ib_opened++;
#ifdef HAS_DMAMANAGER
        if ( !ib_opened ){
	  gpib_dma_size = gpib_default_dma_size;

	  if (gpib_dma_size) {
	    /* Pre-allocate a DMA buffer for I/O */
	    if (gpib_dma_size > MAX_DMA_SIZE)
	      gpib_dma_size = MAX_DMA_SIZE;
		
	    gpib_dma_buffer = (char *)dma_mem_alloc( gpib_dma_size,
						    DMA_CEIL_16M |
						    DMA_SEG_128K |
						    DMA_ALIGN_2 );
	    if (gpib_dma_buffer == NULL) {
	      printk( "gpib: Unable to pre-allocate %d byte DMA buffer",
		     gpib_dma_size );
	      printk( " - using dynamic buffer allocation\n" );
	    } else {
	      printk( "gpib: Pre-allocated %d byte buffer for DMA\n",
		     gpib_dma_size );
	    }
	  }
	}
#endif /* HAS_DMAMANAGER */

       /* if implicit adressing mode initialize the bus if necessary */

       if( MINOR(inode->i_rdev) > 0 ) {
           osLockMutex(); /* let complete commands first */
	   if ( !(pgmstat & PS_ONLINE) )
	     ibonl(1);
           ibsic();
           ibsre(1);
           if( ibsta & ERR ){
             ib_opened--;
             MOD_DEC_USE_COUNT;
	     GIVE_UP(-EIO);
	   }
           osUnlockMutex();
       }

       DBGout();
       return (OK);
}


void ibclose(LINUX_STD_PARAM)
{
	DBGin("ibclose");

       /* if implicit adressing mode  unset ren */

       if( (MINOR(inode->i_rdev) > 0 ) ) {
	 ibsre(0);
       }
       if ((pgmstat & PS_ONLINE) && ib_opened == 1 )
		ibonl(0);

#ifdef HAS_DMAMANAGER
        if( ib_opened == 1 ){
	  if (gpib_dma_buffer) {
	    dma_mem_free( (long) gpib_dma_buffer );
	    gpib_dma_buffer = NULL;
	  }
        }
#endif /* HAS_DMAMANAGER */

	ib_opened--;
	MOD_DEC_USE_COUNT;


	DBGout();
}


ibioctl(LINUX_STD_PARAM,unsigned int cmd, unsigned long arg)
{
	int	retval = OK;		/* assume everything OK for now */
        ibarg_t m_ibarg,*ibargp;

#ifndef HAS_DMAMANAGER
	char *buf;
	char c;
#else
	int	bufsize;
	int	remain;
	char 	*buf;
	char 	*userbuf;
	char 	c;
#endif /* HAS_DMAMANAGER */


	DBGin("ibioctl");
        DBGprint(DBG_DATA,("cmd=%d",cmd));
	ibargp = (ibarg_t *) &m_ibarg;

	/* Check the arg buffer is readable & writable by the current process */
	retval = verify_area(VERIFY_WRITE, (void *)arg, sizeof(ibarg_t));
	if (retval)
	  GIVE_UP (retval);

	retval = verify_area(VERIFY_READ, (void *)arg, sizeof(ibarg_t));
	if (retval)
	  GIVE_UP (retval);

        
	memcpy_fromfs( (ibarg_t *) ibargp , (ibarg_t *) arg , sizeof(ibarg_t));

	if( cmd == IBAPWAIT ){ 
          DBGprint(DBG_BRANCH,("IBAPWAIT called"));
        /* special case for IBAPWAIT : does his own locking */
          ibAPWait(ibargp->ib_arg);
	  ibargp->ib_ibsta = ibsta;
	  ibargp->ib_iberr = iberr;
	  ibargp->ib_ibcnt = ibcnt;
   	  memcpy_tofs((ibarg_t *) arg, (ibarg_t *) ibargp , sizeof(ibarg_t));

	  DBGout();
	  return retval;
	}

	osLockMutex();        /* lock other processes from performing commands */
                              /* quick & dirty hack (glenn will flame me :-)  */
	switch (cmd) {

	case IBRD:

 	  /* Check write access to buffer */
 	  retval = verify_area(VERIFY_WRITE, ibargp->ib_buf, ibargp->ib_cnt);
 	  if (retval)
 	    GIVE_UP (retval);
 	  

#ifndef HAS_DMAMANAGER
	  if ((buf = kmalloc( ibargp->ib_cnt +1 , MEMORY_PRIORITY )) == NULL) {
#else /* HAS_DMA */
	  /* Get a DMA buffer */
	  bufsize = ibargp->ib_cnt;
	  if ((buf = getdmabuffer( &bufsize )) == NULL) {
#endif /* HAS_DMAMANAGER */
		  DBGout();
		  GIVE_UP( -ENOMEM ) ;
	  }
#ifndef HAS_DMAMANAGER
	  ibrd(buf, ibargp->ib_cnt);
	  memcpy_tofs( ibargp->ib_buf, buf, ibcnt );
	  kfree_s( buf, ibargp->ib_cnt +1 );
#else
	  
	  /* Read DMA buffer loads till we fill the user supplied buffer */
	  userbuf = ibargp->ib_buf;
	  remain = ibargp->ib_cnt;
	  do {
		  ibrd( buf, (bufsize < remain) ? bufsize : remain );
		  memcpy_tofs( userbuf, buf, ibcnt );
		  remain -= ibcnt;
		  userbuf += ibcnt;
	  } while (remain > 0 && ibcnt > 0 && !(ibsta & END));
	  ibcnt = ibargp->ib_cnt - remain;

	  /* Free the DMA buffer */
	  freedmabuffer( buf );
#endif /* HAS_DMAMANAGER */

	  break;


	case IBWRT:

 	  /* Check read access to buffer */
 	  retval = verify_area(VERIFY_READ, ibargp->ib_buf, ibargp->ib_cnt);
 	  if (retval)
 	    GIVE_UP(retval);

#ifndef HAS_DMAMANAGER
	  if ((buf = kmalloc( ibargp->ib_cnt +1 , MEMORY_PRIORITY)) == NULL) {
#else
	  /* Get a DMA buffer */
	  bufsize = ibargp->ib_cnt;
	  if ((buf = getdmabuffer( &bufsize )) == NULL) {
#endif /* HAS_DMAMANAGER */
		  DBGout();
		  GIVE_UP(-ENOMEM);
	  }
#ifndef HAS_DMAMANAGER
	  memcpy_fromfs( buf, ibargp->ib_buf, ibargp->ib_cnt );
	  ibwrt(buf, ibargp->ib_cnt);
	  kfree_s( buf, ibargp->ib_cnt +1 );
#else
	  
	  /* Write DMA buffer loads till we empty the user supplied buffer */
	  userbuf = ibargp->ib_buf;
	  remain = ibargp->ib_cnt;
	  do {
		  memcpy_fromfs( buf, userbuf, (bufsize < remain) ? bufsize : remain );
		  ibwrt( buf, (bufsize < remain) ? bufsize : remain );
		  remain -= ibcnt;
		  userbuf += ibcnt;
	  } while (remain > 0 && ibcnt > 0 && !(ibsta & (ERR | TIMO)));
	  ibcnt = ibargp->ib_cnt - remain;

	  /* Free the DMA buffer */
	  freedmabuffer( buf );
#endif /* HAS_DMAMANAGER */

	  break;

	case IBCMD:

 	  /* Check read access to buffer */
 	  retval = verify_area(VERIFY_READ, ibargp->ib_buf, ibargp->ib_cnt);
 	  if (retval)
 	    GIVE_UP(retval);

#ifndef HAS_DMAMANAGER
	  if ((buf = kmalloc( ibargp->ib_cnt +1 , MEMORY_PRIORITY )) == NULL) {
#else
	  /* Get a DMA buffer */
	  bufsize = ibargp->ib_cnt;
	  if ((buf = getdmabuffer( &bufsize )) == NULL) {
#endif /* HAS_DMAMANAGER */
		  DBGout();
		  GIVE_UP(-ENOMEM);
	  }
#ifndef HAS_DMAMANAGER
	  memcpy_fromfs( buf, ibargp->ib_buf, ibargp->ib_cnt );
	  ibcmd(buf, ibargp->ib_cnt);
	  kfree_s( buf, ibargp->ib_cnt +1 );
#else
	  
	  /* Write DMA buffer loads till we empty the user supplied buffer */
	  userbuf = ibargp->ib_buf;
	  remain = ibargp->ib_cnt;
	  do {
		  memcpy_fromfs( buf, userbuf, (bufsize < remain) ? bufsize : remain );
		  ibcmd( buf, (bufsize < remain) ? bufsize : remain );
		  remain -= ibcnt;
		  userbuf += ibcnt;
	  } while (remain > 0 && ibcnt > 0 && !(ibsta & (ERR | TIMO)));
	  ibcnt = ibargp->ib_cnt - remain;

	  /* Free the DMA buffer */
	  freedmabuffer( buf );
#endif /* HAS_DMAMANAGER */

	  break;

	case IBWAIT:
	  DBGprint(DBG_DATA,("**arg=%x",ibargp->ib_arg));
	  ibwait(ibargp->ib_arg);
	  break;
	case IBRPP:
	  /* Check write access to Poll byte */
	  retval = verify_area(VERIFY_WRITE, ibargp->ib_buf, 1);
	  if (retval)
	    GIVE_UP(retval);
	  
	  ibrpp(&c);
	  put_fs_byte( c, ibargp->ib_buf );
	  break;

	case IBONL:
	  ibonl(ibargp->ib_arg);
	  break;
	case IBAPE:
	  ibAPE(ibargp->ib_arg,ibargp->ib_cnt);
	  break;
	case IBSIC:
	  ibsic();
	  break;
	case IBSRE:
	  ibsre(ibargp->ib_arg);
	  break;
	case IBGTS:
	  ibgts();
	  break;
	case IBCAC:
	  ibcac(ibargp->ib_arg);
	  break;
	case IBPOKE:
	  ibpoke(ibargp->ib_arg);
	  break;
	case IBLINES:
	  iblines(&ibargp->ib_ret);
	  break;
	case IBPAD:
	  ibpad(ibargp->ib_arg);
	  break;
	case IBSAD:
	  ibsad(ibargp->ib_arg);
	  break;
	case IBTMO:
	  ibtmo(ibargp->ib_arg);
	  break;
	case IBEOT:
	  ibeot(ibargp->ib_arg);
	  break;
	case IBEOS:
	  ibeos(ibargp->ib_arg);
	  break;
	case IBRSV:
	  ibrsv(ibargp->ib_arg);
	  break;
	case DVTRG:
	  dvtrg(ibargp->ib_arg);
	  break;
	case DVCLR:
	  dvclr(ibargp->ib_arg);
	  break;

	case DVRSP:

	  /* Check write access to Poll byte */
	  retval = verify_area(VERIFY_WRITE, ibargp->ib_buf, 1);
	  if (retval){
	    GIVE_UP(retval);
	  }
	  dvrsp(ibargp->ib_arg, &c);

	  put_fs_byte( c, ibargp->ib_buf );

	  break;
        case IBAPRSP:
	  retval = verify_area(VERIFY_WRITE, ibargp->ib_buf, 1);
	  if (retval){
	    GIVE_UP(retval);
	  }
	  ibAPrsp(ibargp->ib_arg, &c);
	  put_fs_byte( c, ibargp->ib_buf );
	  break;
	case DVRD:

 	  /* Check write access to buffer */
 	  retval = verify_area(VERIFY_WRITE, ibargp->ib_buf, ibargp->ib_cnt);
 	  if (retval)
 	    GIVE_UP(retval);

#ifndef HAS_DMAMANAGER
	  if ((buf = kmalloc( ibargp->ib_cnt +1, MEMORY_PRIORITY )) == NULL) {
#else
	  /* Get a DMA buffer */
	  bufsize = ibargp->ib_cnt;
	  if ((buf = getdmabuffer( &bufsize )) == NULL) {
#endif /* HAS_DMAMANAGER */
		  DBGout();
		  GIVE_UP(-ENOMEM);
	  }
#ifndef HAS_DMAMANAGER
	  dvrd(ibargp->ib_arg, buf, ibargp->ib_cnt);
	  memcpy_tofs( ibargp->ib_buf, buf, ibcnt );
	  kfree_s( buf, ibargp->ib_cnt +1);
#else
	  
	  /* Read DMA buffer loads till we fill the user supplied buffer */
	  userbuf = ibargp->ib_buf;
	  remain = ibargp->ib_cnt;
	  do {
		  dvrd( ibargp->ib_arg, buf, (bufsize < remain) ? bufsize : remain );
		  memcpy_tofs( userbuf, buf, ibcnt );
		  remain -= ibcnt;
		  userbuf += ibcnt;
	  } while (remain > 0 && ibcnt > 0 && !(ibsta & END));
	  ibcnt = ibargp->ib_cnt - remain;

	  /* Free the DMA buffer */
	  freedmabuffer( buf );
#endif /* HAS_DMAMANAGER */

	  break;

	case DVWRT:

 	  /* Check read access to buffer */
 	  retval = verify_area(VERIFY_READ, ibargp->ib_buf, ibargp->ib_cnt);
 	  if (retval)
 	    GIVE_UP(retval);

#ifndef HAS_DMAMANAGER
	  if ((buf = kmalloc( ibargp->ib_cnt +1, MEMORY_PRIORITY )) == NULL) {
#else
	  /* Get a DMA buffer */
	  bufsize = ibargp->ib_cnt;
	  if ((buf = getdmabuffer( &bufsize )) == NULL) {
#endif /* HAS_DMAMANAGER */
		  DBGout();
		  GIVE_UP(-ENOMEM);
	  }


#ifndef HAS_DMAMANAGER
	  memcpy_fromfs( (char *) buf, (char *) ibargp->ib_buf, ibargp->ib_cnt );
	  dvwrt(ibargp->ib_arg, buf, ibargp->ib_cnt);
	  kfree_s( buf, ibargp->ib_cnt +1);
#else
	  
	  /* Write DMA buffer loads till we empty the user supplied buffer */
	  userbuf = ibargp->ib_buf;
	  remain = ibargp->ib_cnt;
	  do {
		  memcpy_fromfs( buf, userbuf, (bufsize < remain) ? bufsize : remain );
		  dvwrt( ibargp->ib_arg, buf, (bufsize < remain) ? bufsize : remain );
		  remain -= ibcnt;
		  userbuf += ibcnt;
	  } while (remain > 0 && ibcnt > 0 && !(ibsta & (ERR | TIMO)));
	  ibcnt = ibargp->ib_cnt - remain;

	  /* Free the DMA buffer */
	  freedmabuffer( buf );
#endif /* HAS_DMAMANAGER */

	  break;

	  /* special configuration options */
	case CFCBASE:
	  osChngBase(ibargp->ib_arg);
	  break;
	case CFCIRQ:
	  osChngIRQ(ibargp->ib_arg);
	  break;
	case CFCDMA:
	  osChngDMA(ibargp->ib_arg);
#ifdef HAS_DMAMANAGER
        case CFCDMABUFFER:
	  if (ibargp->ib_arg > MAX_DMA_SIZE) {
		  DBGout();
		  GIVE_UP(-EINVAL);
	  }
			  
#ifdef CONFIG_DMA_MEM_PERM
	  if (gpib_dma_buffer)
		  dma_mem_free( (long) gpib_dma_buffer );

	  gpib_dma_buffer = (char *)dma_mem_alloc( ibargp->ib_arg,
						  DMA_CEIL_16M |
						  DMA_SEG_128K |
						  DMA_ALIGN_2);
#else
	  if (gpib_dma_buffer)
		  kfree( gpib_dma_buffer );

	  gpib_dma_buffer = (char *)kmalloc( ibargp->ib_arg, MEMORY_PRIORITY );
#endif
	  
	  if (gpib_dma_buffer == NULL) {
		  DBGout();
		  GIVE_UP(-ENOMEM);
	  }
	  gpib_dma_size = ibargp->ib_arg;
	  
#endif /* HAS_DMAMANAGER */
	  break;

	default:
	  DBGprint(DBG_DATA,("No such Command : %d",cmd));
	  retval = -EINVAL;
	}
	ibargp->ib_ibsta = ibsta;
	ibargp->ib_iberr = iberr;
	ibargp->ib_ibcnt = ibcnt;
	memcpy_tofs((ibarg_t *) arg, (ibarg_t *) ibargp , sizeof(ibarg_t));

	GIVE_UP(retval);
}



/***********************************************************************/

ibVFSwrite( LINUX_STD_PARAM, char *buffer, int count)
  {
    int minor = MINOR(inode->i_rdev);
    int retval = 0;
#ifndef HAS_DMAMANAGER
	char *buf;
	char c;
#else
	int	bufsize;
	int	remain;
	char 	*buf;
	char 	*userbuf;
	char 	c;
#endif /* HAS_DMAMANAGER */


    DBGin("ibVFSwrite");

    osLockMutex();


    /* Check read access to buffer */
    retval = verify_area(VERIFY_READ, buffer, count);
    if (retval)
 	    GIVE_UP(retval);

#ifndef HAS_DMAMANAGER
	  if ((buf = kmalloc( count +1, MEMORY_PRIORITY )) == NULL) {
#else
	  /* Get a DMA buffer */
	  bufsize = count;
	  if ((buf = getdmabuffer( &bufsize )) == NULL) {
#endif /* HAS_DMAMANAGER */
		  DBGout();
		  GIVE_UP(-ENOMEM);
	  }


#ifndef HAS_DMAMANAGER
	  memcpy_fromfs( (char *) buf, (char *) buffer, count );
	  dvwrt( minor , buf, count);
	  kfree_s( buf, count +1);
#else
	  
	  /* Write DMA buffer loads till we empty the user supplied buffer */
	  userbuf = buffer;
	  remain = count;
	  do {
		  memcpy_fromfs( buf, userbuf, (bufsize < remain) ? bufsize : remain );
		  dvwrt( minor , buf, (bufsize < remain) ? bufsize : remain );
		  remain -= ibcnt;
		  userbuf += ibcnt;
	  } while (remain > 0 && ibcnt > 0 && !(ibsta & (ERR | TIMO)));
	  ibcnt = count - remain;

	  /* Free the DMA buffer */
	  freedmabuffer( buf );
#endif /* HAS_DMAMANAGER */

    GIVE_UP(ibcnt);
  }



/*----------------------------------------------------------------------*/

ibVFSread( LINUX_STD_PARAM, char *buffer, int count)
  {
    int minor = MINOR(inode->i_rdev);
    int retval = 0;

#ifndef HAS_DMAMANAGER
	char *buf;
	char c;
#else
	int	bufsize;
	int	remain;
	char 	*buf;
	char 	*userbuf;
	char 	c;
#endif /* HAS_DMAMANAGER */



    DBGin("ibVFSwrite");

    osLockMutex();


    /* Check write access to buffer */
    retval = verify_area(VERIFY_WRITE, buffer, count);
    if (retval)
 	    GIVE_UP(retval);

#ifndef HAS_DMAMANAGER
    if ((buf = kmalloc( count +1, MEMORY_PRIORITY )) == NULL) {
#else
	  /* Get a DMA buffer */
	  bufsize = count;
	  if ((buf = getdmabuffer( &bufsize )) == NULL) {
#endif /* HAS_DMAMANAGER */
		  DBGout();
		  GIVE_UP(-ENOMEM);
	  }




#ifndef HAS_DMAMANAGER
	  dvrd( minor , buf, count);
	  memcpy_tofs( buffer , buf, ibcnt );
	  kfree_s( buf, count +1);
#else
	  
	  /* Read DMA buffer loads till we fill the user supplied buffer */
	  userbuf = buffer;
	  remain = count;
	  do {
		  dvrd( minor , buf, (bufsize < remain) ? bufsize : remain );
		  memcpy_tofs( userbuf, buf, ibcnt );
		  remain -= ibcnt;
		  userbuf += ibcnt;
	  } while (remain > 0 && ibcnt > 0 && !(ibsta & END));
	  ibcnt = count - remain;

	  /* Free the DMA buffer */
	  freedmabuffer( buf );
#endif /* HAS_DMAMANAGER */



    GIVE_UP(ibcnt);
  }

