					/* ... other system includes?	*/

typedef char		int8;		/* HW-independent data types...	*/
typedef unsigned char	uint8;
typedef short		int16;
typedef unsigned short	uint16;
typedef long		int32;
typedef unsigned long	uint32;

typedef unsigned char	*faddr_t;	/* far character pointer	*/

					/* DMA controller access macros	*/
#define DMAin(x)	osP8in(&dmac->x)
#define DMAout(x,v)	osP8out(&dmac->x, v)

#define GPIBin(x)	bdP8in(&IB->x)	/* Board access macros...	*/
#define GPIBout(x,v)	bdP8out(&IB->x, v)

#define GPIBin16(x)	bdP16in(&IB->x)
#define GPIBout16(x,v)	bdP16out(&IB->x, v)

#define GPIBpgin(x)	(GPIBout(auxmr, AUX_PAGE), GPIBin(x))
#define GPIBpgout(x,v)	(GPIBout(auxmr, AUX_PAGE), GPIBout(x,v))


/*
 * I/O operation parameters
 */
#define	IO_READ		(1 << 0)	/* 1: I/O read; 0: I/O write	*/
#define	IO_NOHLDA	(1 << 1)	/* don't do HLDA carry-cycle	*/
#define	IO_NOEOI	(1 << 2)	/* don't send EOI w/last byte	*/
#define	IO_LAST		(1 << 3)	/* last "chunk" in I/O transfer	*/

typedef struct ibio_op {
	faddr_t		io_vbuf;	/* virtual buffer address	*/
	uint32		io_pbuf;	/* physical buffer address	*/
	unsigned	io_cnt;		/* transfer count		*/
	int		io_flags;	/* direction flags, etc.	*/
	uint8		io_ccfunc;	/* carry-cycle function		*/
} ibio_op_t;


					/* 8237A-5 MODE BITS...         */
#define MO_WRITEM	0x04		/* DMA to system memory         */
#define MO_READM	0x08		/* DMA from system memory       */
#define MO_AUTOINIT	0x10		/* Auto-initialization after TC */
#define MO_SINGLE	0x40		/* One xfer only (cycle-steal)  */
#define MO_DEMAND	0x00		/* Continuous xfers as possible */
#define MO_SETMASK	0x04		/* Set mask bit (disable DMAC)  */


#define IBDMABASE	4		/* DMA channel base value       */

typedef struct dmachan {		/* 8237A-5 DMA CONTROLLER...    */
	uint8		d_bar;		/* base/current address regs    */
	uint8	pad1,	d_bcr;		/* base/current byte count regs */
	uint8	pad2;
} dmachan_t;

typedef struct dma {
	dmachan_t	d_ch[4];	/* DMA channels 4, 5, 6, 7      */
	uint8		d_csr;		/* command/status register      */
	uint8	pad1,	d_req;		/* request register             */
	uint8	pad2,	d_smask;	/* set/clear mask register bit  */
	uint8	pad3,	d_mode;		/* channel mode select register */
	uint8	pad4,	d_bp0;		/* byte pointer reset register  */
	uint8	pad5,	d_mr;		/* master reset/temporary reg   */
	uint8	pad6,	d_clrm;		/* clear mask register          */
	uint8	pad7,	d_mask;		/* channel mask register        */
	uint8	pad8;
} dma_t;


typedef union {				/* FIFO access type		*/
    struct {
	uint8	b;			/*  +0 8-bit FIFO "B" register	*/
	uint8	a;			/*  +1 8-bit FIFO "A" register	*/

    }		f8;			/* two 8-bit FIFO registers A/B	*/
    uint16	f16;			/* single 16-bit FIFO register	*/

} fifo_t;

#ifdef NIAT
/*
 * Register layout for NI AT-GPIB board
 *
 */
typedef struct ibregs {			/*  hardware registers for AT-GPIB      */
	uint8		cdor;		/*  +0 (x00) byte out register          */
	uint8		gpibc;		/*  +1 (x01) GPIB control lines         */
	uint8		imr1;		/*  +2 (x02) interrupt mask register 1  */
	uint8		gpibd;		/*  +3 (x03) GPIB data lines            */
	uint8		imr2;		/*  +4 (x04) interrupt mask register 2  */
	uint8		keyprt;		/*  +5 (x05) port for key and dmaEn bit */
	uint8		spmr;		/*  +6 (x06) serial poll mode register  */
	uint8		intrt;		/*  +7 (x07) interrupt register         */
	uint8		admr;		/*  +8 (x08) address mode register      */
	uint8	pad5,	auxmr;		/* +10 (x0A) auxiliary mode register    */
	uint8	pad6,	adr;		/* +12 (x0C) address register 0/1       */
	uint8	pad7,	eosr;		/* +14 (x0E) end of string register     */
					/* TURBO488...                          */
	uint8	pad8,	cfg;		/* +16 (x10) configuration register     */
	uint8	pad9,	imr3;		/* +18 (x12) interrupt mask register 3  */
	uint8	pad10,	cntl;		/* +20 (x14) byte count low register    */
	uint8	pad11,	cnth;		/* +22 (x16) byte count high register   */
	uint8	pad12;			/*           PADDING                    */
	fifo_t		fifo;		/* +24 (x18) 16-bit FIFO register (B/A) */
	uint8		ccrg;		/* +26 (x1A) carry cycle register       */
	uint8	pad13,	cmdr;		/* +28 (x1C) command register           */
	uint8	pad14,	timer;		/* +30 (x1E) timer register             */

} ibregs_t;

#endif

#ifdef NIPCII 

/*
 *  Register layout for PCII and PCIIa (in PCII mode :-) )
 *
 */

typedef struct ibregs {			/*  hardware registers for PCII      */
	uint8		cdor;		/*  (x00) byte out register          */
	uint8		imr1;		/*  (x01) interrupt mask register 1  */
	uint8		imr2;		/*  (x02) interrupt mask register 2  */
	uint8		spmr;		/*  (x03) serial poll mode register  */
	uint8		admr;		/*  (x04) address mode register      */
	uint8		auxmr;		/*  (x05) auxiliary mode register    */
	uint8		adr;		/*  (x06) address register 0/1       */
	uint8		eosr;		/*  (x07) end of string register     */
} ibregs_t;

#endif

/* Read-only register mnemonics corresponding to write-only registers */

#define dir		cdor		/* Data In Register */
#define isr1		imr1		/* Interrupt Status Register 1 */
#define isr2		imr2		/* Interrupt Status Register 2 */
#define spsr		spmr		/* Serial Poll Status Register */
#define adsr		admr		/* Address Status Register */
#define cptr		auxmr		/* Command Pass Through Register */
#define adr0		adr		/* Address 0 Register */
#define adr1		eosr		/* Address 1 Register */ 

#ifdef NIAT

#define sts1		cfg		/* T488 Status Register 1 */
#define sts2		cmdr	        /* T488 Status Register 2 */
#define isr3		ccrg		/* T488 Interrupt Status Register 3 */
#define dmaEn		keyprt		/* DMA Enable Register (Key Port) */

#endif

/* NAT4882 "Paged In" read- and write-only registers (must use GPIBpg__ macros) */

#define csr		spmr		/* Chip Signature Register */
#define keyr		spmr		/* Key Register */
#define sasr		auxmr		/* Source/Acceptor Status Register */
#define isr0		adr		/* Interrupt Status Register 0 */
#define imr0		adr		/* Interrupt Mask Register 0 */
#define bsr		eosr		/* Bus Status Register */ 
#define bcr		eosr		/* Bus Control Register */ 


/*--------------------------------------------------------------*/
/* 7210 bits:                POSITION              7210 reg     */
/*--------------------------------------------------------------*/
#define HR_DI           (unsigned char)(1<<0)	/* ISR1         */
#define HR_DO           (unsigned char)(1<<1)	/*  ,           */
#define HR_ERR          (unsigned char)(1<<2)	/*  ,           */
#define HR_DEC          (unsigned char)(1<<3)	/*  ,           */
#define HR_END          (unsigned char)(1<<4)	/*  ,           */
#define HR_DET          (unsigned char)(1<<5)	/*  ,           */
#define HR_APT          (unsigned char)(1<<6)	/*  ,           */
#define HR_CPT          (unsigned char)(1<<7)	/*  ,           */
#define HR_DIIE         (unsigned char)(1<<0)	/* IMR1         */
#define HR_DOIE         (unsigned char)(1<<1)	/*  ,           */
#define HR_ERRIE        (unsigned char)(1<<2)	/*  ,           */
#define HR_DECIE        (unsigned char)(1<<3)	/*  ,           */
#define HR_ENDIE        (unsigned char)(1<<4)	/*  ,           */
#define HR_DETIE        (unsigned char)(1<<5)	/*  ,           */
#define HR_APTIE        (unsigned char)(1<<6)	/*  ,           */
#define HR_CPTIE        (unsigned char)(1<<7)	/*  ,           */
#define HR_ADSC         (unsigned char)(1<<0)	/* ISR2         */
#define HR_REMC         (unsigned char)(1<<1)	/*  ,           */
#define HR_LOKC         (unsigned char)(1<<2)	/*  ,           */
#define HR_CO           (unsigned char)(1<<3)	/*  ,           */
#define HR_REM          (unsigned char)(1<<4)	/*  ,           */
#define HR_LOK          (unsigned char)(1<<5)	/*  ,           */
#define HR_SRQI         (unsigned char)(1<<6)	/*  ,           */
#define HR_INT          (unsigned char)(1<<7)	/*  ,           */
#define HR_ACIE         (unsigned char)(1<<0)	/* IMR2         */
#define HR_REMIE        (unsigned char)(1<<1)	/*  ,           */
#define HR_LOKIE        (unsigned char)(1<<2)	/*  ,           */
#define HR_COIE         (unsigned char)(1<<3)	/*  ,           */
#define HR_DMAI         (unsigned char)(1<<4)	/*  ,           */
#define HR_DMAO         (unsigned char)(1<<5)	/*  ,           */
#define HR_SRQIE        (unsigned char)(1<<6)	/*  ,           */
#define HR_PEND         (unsigned char)(1<<6)	/* SPSR         */
#define HR_RSV          (unsigned char)(1<<6)	/* SPMR         */
#define HR_MJMN         (unsigned char)(1<<0)	/* ADSR         */
#define HR_TA           (unsigned char)(1<<1)	/*  ,           */
#define HR_LA           (unsigned char)(1<<2)	/*  ,           */
#define HR_TPAS         (unsigned char)(1<<3)	/*  ,           */
#define HR_LPAS         (unsigned char)(1<<4)	/*  ,           */
#define HR_SPMS         (unsigned char)(1<<5)	/*  ,           */
#define HR_NATN         (unsigned char)(1<<6)	/*  ,           */
#define HR_CIC          (unsigned char)(1<<7)	/*  ,           */
#define HR_ADM0         (unsigned char)(1<<0)	/* ADMR         */
#define HR_ADM1         (unsigned char)(1<<1)	/*  ,           */
#define HR_TRM0         (unsigned char)(1<<4)	/*  ,           */
#define HR_TRM1         (unsigned char)(1<<5)	/*  ,           */
#define HR_LON          (unsigned char)(1<<6)	/*  ,           */
#define HR_TON          (unsigned char)(1<<7)	/*  ,           */
#define HR_DL           (unsigned char)(1<<5)	/* ADR          */
#define HR_DT           (unsigned char)(1<<6)	/*  ,           */
#define HR_ARS          (unsigned char)(1<<7)	/*  ,           */
#define HR_EOI          (unsigned char)(1<<7)	/* ADR1         */
#define HR_HLDA         (unsigned char)(1<<0)	/* auxra        */
#define HR_HLDE         (unsigned char)(1<<1)	/*  ,           */
#define HR_REOS         (unsigned char)(1<<2)	/*  ,           */
#define HR_XEOS         (unsigned char)(1<<3)	/*  ,           */
#define HR_BIN          (unsigned char)(1<<4)	/*  ,           */
#define HR_CPTE         (unsigned char)(1<<0)	/* auxrb        */
#define HR_SPEOI        (unsigned char)(1<<1)	/*  ,           */
#define HR_TRI          (unsigned char)(1<<2)	/*  ,           */
#define HR_INV          (unsigned char)(1<<3)	/*  ,           */
#define HR_ISS          (unsigned char)(1<<4)	/*  ,           */
#define HR_PPS          (unsigned char)(1<<3)	/* ppr          */
#define HR_PPU          (unsigned char)(1<<4)	/*  ,           */

#define HR_LCM (unsigned char)(HR_HLDE|HR_HLDA)	/* auxra listen continuous */

#define CR_EOI		(unsigned char)(1<<7)	/* gpibc        */
#define CR_ATN		(unsigned char)(1<<6)	/*  ,           */
#define CR_SRQ		(unsigned char)(1<<5)	/*  ,           */
#define CR_REN		(unsigned char)(1<<4)	/*  ,           */
#define CR_IFC		(unsigned char)(1<<3)	/*  ,           */
#define CR_NRFD		(unsigned char)(1<<2)	/*  ,           */
#define CR_NDAC		(unsigned char)(1<<1)	/*  ,           */
#define CR_DAV		(unsigned char)(1<<0)	/*  ,           */

#define BR_ATN		(unsigned char)(1<<7)	/* bsr and bcr  */
#define BR_DAV		(unsigned char)(1<<6)	/*  ,           */
#define BR_NDAC		(unsigned char)(1<<5)	/*  ,           */
#define BR_NRFD		(unsigned char)(1<<4)	/*  ,           */
#define BR_EOI		(unsigned char)(1<<3)	/*  ,           */
#define BR_SRQ		(unsigned char)(1<<2)	/*  ,           */
#define BR_IFC		(unsigned char)(1<<1)	/*  ,           */
#define BR_REN		(unsigned char)(1<<0)	/*  ,           */

#define HR_nba		(unsigned char)(1<<7)	/* sasr         */
#define HR_AEHS		(unsigned char)(1<<6)	/*  ,           */
#define HR_ANHS1	(unsigned char)(1<<5)	/*  ,           */
#define HR_ANHS2	(unsigned char)(1<<4)	/*  ,           */
#define HR_ADHS		(unsigned char)(1<<3)	/*  ,           */
#define HR_ACRDY	(unsigned char)(1<<2)	/*  ,           */
#define HR_SH1A		(unsigned char)(1<<1)	/*  ,           */
#define HR_SH1B		(unsigned char)(1<<0)	/*  ,           */


#define ICR		(unsigned char)0040	/* AUXMR control masks for hidden regs */
#define PPR		(unsigned char)0140
#define AUXRA		(unsigned char)0200
#define AUXRB		(unsigned char)0240
#define AUXRE		(unsigned char)0300
#define LOMASK		0x1F			/* mask to specify lower 5 bits */

/* 7210 Auxiliary Commands */
#define AUX_PON         (unsigned char)000	/* Immediate Execute pon                  */
#define AUX_CPPF        (unsigned char)001	/* Clear Parallel Poll Flag               */
#define AUX_CR          (unsigned char)002	/* Chip Reset                             */
#define AUX_FH          (unsigned char)003	/* Finish Handshake                       */
#define AUX_TRIG        (unsigned char)004	/* Trigger                                */
#define AUX_RTL         (unsigned char)005	/* Return to local                        */
#define AUX_SEOI        (unsigned char)006	/* Send EOI                               */
#define AUX_NVAL        (unsigned char)007	/* Non-Valid Secondary Command or Address */
#define AUX_SPPF        (unsigned char)011	/* Set Parallel Poll Flag                 */
#define AUX_VAL         (unsigned char)017	/* Valid Secondary Command or Address     */
#define AUX_TCA         (unsigned char)021	/* Take Control Asynchronously            */
#define AUX_GTS         (unsigned char)020	/* Go To Standby                          */
#define AUX_TCS         (unsigned char)022	/* Take Control Synchronously             */
#define AUX_LTN         (unsigned char)023	/* Listen                                 */
#define AUX_DSC         (unsigned char)024	/* Disable System Control                 */
#define AUX_CIFC        (unsigned char)026	/* Clear IFC                              */
#define AUX_CREN        (unsigned char)027	/* Clear REN                              */
#define AUX_TCSE        (unsigned char)032	/* Take Control Synchronously on End      */
#define AUX_LTNC        (unsigned char)033	/* Listen in Continuous Mode              */
#define AUX_LUN         (unsigned char)034	/* Local Unlisten                         */
#define AUX_EPP         (unsigned char)035	/* Execute Parallel Poll                  */
#define AUX_SIFC        (unsigned char)036	/* Set IFC                                */
#define AUX_SREN        (unsigned char)037	/* Set REN                                */

#define AUX_PAGE	(unsigned char)0x50	/* Page-In additional registers           */	


/* TURBO-488 registers bit definitions */

/* STS1 -- Status Register 1 (read only) */
#define S_DONE          (unsigned char)(0x80)	/* DMA done                           */
#define S_SC            (unsigned char)(0x40)	/* is system contoller                */
#define S_IN            (unsigned char)(0x20)	/* DMA in (to memory)                 */
#define S_DRQ           (unsigned char)(0x10)	/* DRQ line (for diagnostics)         */
#define S_STOP          (unsigned char)(0x08)	/* DMA stopped                        */
#define S_NDAV          (unsigned char)(0x04)	/* inverse of DAV                     */
#define S_HALT          (unsigned char)(0x02)	/* status of transfer machine         */
#define S_GSYNC         (unsigned char)(0x01)	/* indicates if GPIB is in sync w I/O */

/* CFG -- Configuration Register (write only) */
#define	C_CMD	        (unsigned char)(1<<7)	/* FIFO 'bcmd' in progress            */
#define	C_TLCH	        (unsigned char)(1<<6)	/* halt DMA on TLC interrupt          */
#define	C_IN	        (unsigned char)(1<<5)	/* DMA is a GPIB read                 */
#define	C_A_B	        (unsigned char)(1<<4)	/* fifo order 1=motorola, 0=intel     */
#define	C_CCEN	        (unsigned char)(1<<3)	/* enable carry cycle                 */
#define	C_TMOE	        (unsigned char)(1<<2)	/* enable CPU bus time limit          */
#define	C_T_B	        (unsigned char)(1<<1)  	/* tmot reg is: 1=125ns clocks,       */
						/* 0=num bytes                        */
#define	C_B16	        (unsigned char)(1<<0)  	/* 1=FIFO is 16-bit register, 0=8-bit */

/* ISR3 -- Interrupt Status Register (read only) */
#define	HR_INTR	        (unsigned char)(1<<7)	/* 1=board is interrupting	*/
#define	HR_SRQI_CIC	(unsigned char)(1<<5)	/* SRQ asserted and we are CIC	*/
#define	HR_STOP         (unsigned char)(1<<4)	/* fifo empty or STOP command	*/
						/* issued			*/
#define	HR_NFF	        (unsigned char)(1<<3)	/* NOT full fifo		*/
#define	HR_NEF	        (unsigned char)(1<<2)	/* NOT empty fifo		*/
#define	HR_TLCI	        (unsigned char)(1<<1)	/* TLC interrupt asserted	*/
#define	HR_DONE         (unsigned char)(1<<0)	/* DMA done			*/

/* CMDR -- Command Register */
#define	CLRSC		(unsigned char)0x2	/* clear the SC bit 		*/
#define	SETSC		(unsigned char)0x3	/* set the SC bit 		*/
#define	GO		(unsigned char)(1<<2)	/* start DMA 			*/
#define	STOP		(unsigned char)(1<<3)	/* stop DMA 			*/
#define	RSTFIFO		(unsigned char)(1<<4)	/* reset the FIFO 		*/
#define SFTRST		(unsigned char)(1<<5)	/* issue a software reset 	*/
#define	DU_ADD		(unsigned char)(1<<6)	/* Motorola mode dual 	  	*/
#define	DDU_ADD		(unsigned char)(1<<7)	/* Disable dual addressing 	*/

/* STS2 -- Status Register 2 */
#define AFFN		(unsigned char)(1<<3)	/* "A full FIFO NOT"  (0=FIFO full)  */
#define AEFN		(unsigned char)(1<<2)	/* "A empty FIFO NOT" (0=FIFO empty) */
#define BFFN		(unsigned char)(1<<1)	/* "B full FIFO NOT"  (0=FIFO full)  */
#define BEFN		(unsigned char)(1<<0)	/* "B empty FIFO NOT" (0=FIFO empty) */


/* Miscellaneous Register Definitions */

#define HR_INTEN	(unsigned char)(1<<0)	/* INTRT */
#define HR_DMAEN	(unsigned char)(1<<0)	/* DMA   */


/* PROGRAM STATUS codes (see int pgmstat) */

#define PS_ONLINE	(1 << 0)		/* GPIB interface is online */
#define PS_SYSRDY	(1 << 1)		/* System support functions are initialized */
#define PS_TIMINST	(1 << 2)		/* Watch dog timer is installed and running */
#define PS_HELD		(1 << 3)		/* Handshake holdoff in effect */
#define PS_NOINTS	(1 << 4)		/* Do NOT use interrupts for I/O and waits */
#define PS_NOEOI	(1 << 5)		/* Do NOT send EOI at the end of writes */
#define PS_SAC		(1 << 6)		/* GPIB interface is System Controller */
#define PS_NOEOSEND	(1 << 7)		/* Do NOT set END in ibsta with EOS */
#define PS_SILENT	(1 << 8)		/* Do NOT print any messages from ibpoke */

#define PS_STATIC	(PS_SYSRDY | PS_NOINTS | PS_NOEOSEND | PS_SILENT)


/* IBPOKE codes for undocumented, low-level control */

#define PK_CODE		0x7FFF0000		/* CODE portion of mask */
#define PK_VALUE	0x0000FFFF		/* VALUE portion of mask */

#define PK_DEBUG	0x00000000		/* Turn on debugging if VALUE non-zero */
#define PK_USEINTS	0x00010000		/* Turn on interrupts if VALUE non-zero */
#define PK_IFCDELAY	0x00020000		/* Adjust length of IFC delay */
#define PK_POLLTIMO	0x00030000		/* Adjust POLL timeout index */
#define PK_ENABMEM	0x00040000		/* Enable system memory for DMA xfers (UNUSED on some systems) */
#define PK_EOSMODE	0x00050000		/* Turn off 'END with EOS' mode if VALUE is zero */
#define PK_INFO		0x00100000		/* Turn off poke messages if VALUE is zero */


/* Interrupt support macros */

#define USEINTS		(INTROP && SYSTIMO)	/* interrupts require SYSTIMO */

#if USEINTS
#define WaitingFor(i)	{if (!(pgmstat & PS_NOINTS)) osWaitForInt(i);}
#else
#define WaitingFor(i)
#endif


/* Timeout support macros */

#if SYSTIMO
#define TimedOut()	(!noTimo)		/* for testing within loops... */
#define NotTimedOut()	(noTimo)

#define TMFAC		HZ			/* timing factor for TM() macro */

#else
#define TimedOut()	((noTimo > 0) && !(--noTimo))
#define NotTimedOut()	((noTimo < 0) || ((noTimo > 0) && --noTimo))

#define TMFAC		(HZ * 1000)		/* timing factor for TM() macro */
#endif						/* -- approximate and system dependent! */

#define INITTIMO	(-1)			/* initial timeout setting */
#define TM(f,n,d)	((((f)*(n))/(d))+1)	/* clock ticks or counts per timo value */




/* driver states */


#define DRV_ONLINE   (1<<0)
#define DRV_IFC      (1<<1)
#define DRV_REN      (1<<2)



