

#include <ib.h>
#include <ibP.h>
#include <stdio.h>
#include <rpc/rpc.h>
#include "rgpib.h"
#include "rgpibP.h"

/* 
 * Network handles are integers with UD_REMOTE bit set
 *
 +
 +
 */


r_handle_t RgpibHandles[MAXHANDLES];
int Rhandle_cnt = 0;

/*----------------------------------------------------------------------*/
int ibRemoteFunc (int handle, int code, int arg, char *buf, int cnt)
{
  gpib_request request, *result;
  CLIENT *cl;

  request.handle  = ibGetUd(handle);
  request.request = code;
  request.arg     = arg;
  request.count   = cnt;
  request.buf     = buf;
  


  result = rgpib_dorequest_1 ( &request, ibGetClient(handle) );
  if( result == NULL ) {
    ibXsta |= CMPL | ERR;
    ibXerr = ENWE;
    ibXcnt = errno;
    ibPutMsg("Network Error during request %d (%s) ",code,ibVerbCode(code));
  } else {

    ibXsta = result->ibsta;
    ibXerr = result->iberr;
    ibXcnt = result->ibcnt;

    if( result->ibsta & ERR ){
      ibPutMsg("Network Request Error: '%s' resulted with an error ",ibVerbCode(code));
    } 
    else if(code == IBRD || code==DVRD ) { 
      bcopy(result->buf,buf,ibcnt);
    }


  }
  return ibsta;
}


static char  hostdomain[MAXNETNAMELEN];
static int   ibfind_remote_called = 0;

/*----------------------------------------------------------------------*/
int ibFindRemote ( char *server, char *device ){
int handle,*ud;

struct gpib_request req;
CLIENT *cl;
int i;


if ( ! ibfind_remote_called ){
/* get host.domain */
#if 1
   gethostname(hostdomain,sizeof(hostdomain));
   i = strlen(hostdomain);
   hostdomain[i++] = '.';
   getdomainname(&hostdomain[i],(sizeof(hostdomain)-i));
#else
   getnetname(hostdomain);

#endif
   DB(printf("host: '%s'\n",hostdomain));
   ibfind_remote_called++;
 }
/* create client stub and create new handle */ 

   if(( handle = ibCreateClientStub(server)) & ERR )  return ERR;

/* remote call gethandle */

   req.buf = device;


/* set auth style */
   cl =  ibGetClient(handle);
   cl->cl_auth = authunix_create(hostdomain,getuid(),getgid(),0,NULL );

   ud = rgpib_gethandle_1( &req , cl );

   /*auth_destroy(cl->cl_auth);*/

   if( ud == NULL ) {
     ibXsta |= CMPL | ERR;
     ibXerr = ENWE;
     ibXcnt = errno;
     return ERR;
   } 

   if( *ud & ERR ){
     ibXsta |= CMPL | ERR ;
     ibXerr = ENWE;
     ibXcnt = errno;
     return ERR;
   }

   /* set remote ud */
   RgpibHandles[(handle & ~UD_REMOTE)].r_ud = *ud;


   return (handle | UD_REMOTE);
}

/*----------------------------------------------------------------------*/
CLIENT *ibGetClient(int handle){
   return ( RgpibHandles[(handle & ~UD_REMOTE)].r_client );
}

int ibGetUd(int handle){
    return ( RgpibHandles[(handle & ~UD_REMOTE)].r_ud);
}
/*----------------------------------------------------------------------*/
int ibCreateClientStub(char *server){

CLIENT *cl;

/* create a client */
cl = clnt_create(server,RGPIBPROG,RGPIBVERS,"tcp");
if ( cl == NULL ){

/* set error code */
ibXerr = ENWE;
ibPutMsg("Network Error: Cant Request Server '%s'",server);

return ERR;
}

if( Rhandle_cnt < MAXHANDLES ){
  /* add the new client to the client table */
  RgpibHandles[Rhandle_cnt].r_client = cl;

  Rhandle_cnt++;
  return ( (Rhandle_cnt-1) | UD_REMOTE);
} else {
  ibXerr = ENTF;
  ibPutMsg("Network Error: No more Handles !");
  return ERR;
}
}
/*----------------------------------------------------------------------*/
char *ibGetNetDB(int ud)
{
  if( ud & UD_REMOTE ){ 
    return NULL;
  }
  return CONF(ud,networkdb);

}
/*----------------------------------------------------------------------*/

int ibCleanupRemote(){
  CLIENT *cl;
  int i;
  for( i=0; i< Rhandle_cnt; i++)
    clnt_destroy(ibGetClient(i));
  Rhandle_cnt = 0;
  ibfind_remote_called = 0;
}

